# Generated by Django 3.0.10 on 2020-10-19 18:01

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("invoices", "0001_initial"),
        ("customers", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="ServiceCategory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        max_length=255, verbose_name="Category name"
                    ),
                ),
            ],
            options={
                "verbose_name": "Service category",
                "verbose_name_plural": "Service categories",
            },
        ),
        migrations.CreateModel(
            name="ServiceItem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True,
                        db_index=True,
                        max_length=255,
                        verbose_name="Service name",
                    ),
                ),
                (
                    "code",
                    models.CharField(
                        blank=True,
                        db_index=True,
                        max_length=255,
                        verbose_name="Code",
                    ),
                ),
                ("price", models.FloatField(verbose_name="Price")),
                (
                    "category",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="items",
                        to="services.ServiceCategory",
                        verbose_name="Category",
                    ),
                ),
            ],
            options={
                "verbose_name": "Service item",
                "verbose_name_plural": "Service items",
                "ordering": ("code",),
            },
        ),
        migrations.CreateModel(
            name="Service",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="Creation date"
                    ),
                ),
                (
                    "qty",
                    models.IntegerField(default=1, verbose_name="Quantity"),
                ),
                ("price", models.FloatField(verbose_name="Price")),
                (
                    "customer",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="services",
                        to="customers.Customer",
                        verbose_name="Customer",
                    ),
                ),
                (
                    "item",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="services.ServiceItem",
                        verbose_name="Service item",
                    ),
                ),
                (
                    "sale",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="services",
                        to="invoices.Sale",
                        verbose_name="Sale",
                    ),
                ),
            ],
            options={
                "verbose_name": "Service",
                "verbose_name_plural": "Service",
            },
        ),
    ]



from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import gettext_lazy as _

from services.models import Service, ServiceCategory


class ServicesService(object):
    def __init__(self, user):
        self._user = user
        self._check_access()

    def get_service(self, service_id):
        try:
            return Service.objects.get(pk=service_id)
        except ObjectDoesNotExist:
            raise Exception(_("Service not found"))

    def add_service(self, sale, service_item):
        try:
            service = Service.objects.get(sale=sale, item=service_item)
            service.qty += 1
        except Service.DoesNotExist:
            service = Service(
                customer=sale.customer,
                sale=sale,
                item=service_item,
                price=service_item.price,
            )

        service.save()

        return service

    def get_categories(self):
        return ServiceCategory.objects.all().prefetch_related("items")

    def _check_access(self):
        if not self._user.is_staff:
            raise Exception(_("Access denied"))



from cap.decorators import admin_render_view
from cap.views import render
from django.contrib.admin.views.decorators import staff_member_required
from django.http.response import (
    HttpResponseBadRequest,
    HttpResponseForbidden,
    JsonResponse,
)
from django.template.loader import render_to_string
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views import View
from django.views.decorators.http import require_GET, require_POST
from djforms import get_clean_data
from pagination import paginate

from invoices.models import Sale
from services import forms
from services.models import Service, ServiceItem


@admin_render_view(template_name="services/print-service-items.html")
def print_service_items(request):
    return {}


@admin_render_view(template_name="services/print-services.html")
def print_services(request, sale_id):
    sale = request.invoices.get_sale(sale_id)
    return {"sale": sale, "services": sale.services.all()}


@admin_render_view(template_name="services/report.html")
def get_report(request):
    form = forms.ReportForm(request.GET)

    services = Service.objects.filter(
        sale__created__date__range=[
            form.cleaned_data["date_from"],
            form.cleaned_data["date_to"],
        ],
        sale__type=Sale.TYPE_CASH_REGISTER,
    ).order_by("sale__created")

    worker = form.cleaned_data.get("worker")

    if worker:
        services = services.filter(worker=worker)

    return {
        "services": services,
        "form": form,
        "totals": {
            "qty": sum([s.qty for s in services]),
            "grand_total": sum([s.subtotal for s in services]),
        },
        **form.cleaned_data,
    }


@staff_member_required
def add_service_item(request):
    form = forms.AddServiceItemForm(request.POST or None)

    status_code = 200

    if request.method == "POST":
        if form.is_valid():
            service_item = form.save()

            return JsonResponse(
                {
                    "message": _("Service added"),
                    "service_item_id": service_item.pk,
                }
            )
        else:
            status_code = 403

    return render(
        request,
        "services/add-service-item.html",
        {"form": form, "status_code": status_code},
        status=status_code,
    )


@require_GET
@staff_member_required
def get_service_items(request):
    form = forms.SearchServiceForm(request.GET)

    page = paginate(
        request,
        ServiceItem.objects.search(**get_clean_data(form)),
        per_page=30,
    )

    return JsonResponse(
        {
            "items": render_to_string(
                "services/service-items.html",
                {"request": request, "page_obj": page},
            ),
            **page.serialize(),
        }
    )


@require_POST
def add_service(request, sale_id):
    sale = request.invoices.get_sale(sale_id)

    service_item = ServiceItem.objects.get(pk=request.POST["service_item"])

    service = request.services.add_service(sale, service_item)

    return JsonResponse(
        {
            "message": _("Service added"),
            "status": "OK",
            "html": service.render(),
            "item_id": service.id,
            "total": sale.serialize_totals(),
        }
    )


@method_decorator(staff_member_required, "dispatch")
class ServiceAPI(View):
    def post(self, request, service_id):
        service = request.services.get_service(service_id)

        try:
            service.update(request.POST)
        except ValueError as e:
            return HttpResponseBadRequest(str(e))

        return JsonResponse(
            {
                "message": _("Service updated"),
                "total": service.sale.serialize_totals(),
            }
        )

    def delete(self, request, service_id):
        if not request.user.is_superuser:
            return HttpResponseForbidden(_("You can`t delete service items"))

        service = request.services.get_service(service_id)

        sale = service.sale

        service.delete()

        return JsonResponse(
            {"message": _("Service removed"), "total": sale.serialize_totals()}
        )



from django import forms
from django.db import models, transaction
from django.template.loader import render_to_string
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from services.managers import ServiceItemManager


class ServiceCategory(models.Model):
    name = models.CharField(_("Category name"), max_length=255)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("Service category")
        verbose_name_plural = _("Service categories")


class ServiceWorker(models.Model):
    name = models.CharField(_("Worker name"), max_length=255)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("Service worker")
        verbose_name_plural = _("Service workers")


class ServiceItem(models.Model):
    category = models.ForeignKey(
        ServiceCategory,
        on_delete=models.CASCADE,
        related_name="items",
        verbose_name=_("Category"),
    )

    name = models.CharField(
        _("Service name"), max_length=255, blank=True, db_index=True
    )

    code = models.CharField(
        _("Code"), max_length=255, blank=True, db_index=True
    )

    price = models.FloatField(_("Price"))

    objects = ServiceItemManager()

    def __str__(self):
        return self.name

    class Meta:
        ordering = ("code",)
        verbose_name = _("Service item")
        verbose_name_plural = _("Service items")


class ServiceRowForm(forms.Form):
    worker = forms.ModelChoiceField(
        queryset=ServiceWorker.objects.all(), required=False
    )

    def __init__(self, record):
        super().__init__(initial={"worker": record.worker_id})
        worker_widget = self.fields["worker"].widget
        worker_widget.attrs["data-url"] = record.api_url
        worker_widget.attrs["data-role"] = "worker-select"
        worker_widget.attrs["class"] = "form-control"


class Service(models.Model):
    sale = models.ForeignKey(
        "invoices.Sale",
        on_delete=models.PROTECT,
        related_name="services",
        verbose_name=_("Sale"),
        blank=True,
        null=True,
    )

    customer = models.ForeignKey(
        "customers.Customer",
        on_delete=models.PROTECT,
        related_name="services",
        verbose_name=_("Customer"),
        blank=True,
        null=True,
    )

    worker = models.ForeignKey(
        ServiceWorker,
        on_delete=models.PROTECT,
        related_name="services",
        verbose_name=_("Worker"),
        blank=True,
        null=True,
    )

    item = models.ForeignKey(
        ServiceItem, on_delete=models.PROTECT, verbose_name=_("Service item")
    )

    created = models.DateTimeField(_("Creation date"), auto_now_add=True)

    qty = models.IntegerField(_("Quantity"), default=1)

    price = models.FloatField(_("Price"))

    def __str__(self):
        if self.customer:
            return "{} - {}".format(self.item.name, self.customer.name)

        return self.item.name

    @transaction.atomic
    def update(self, data):
        if "worker" in data:
            self.worker = ServiceWorker.objects.get(pk=data["worker"])

        if "qty" in data:
            self.qty = float(data["qty"])

        if "price" in data:
            self.price = float(data["price"])

        self.save()

    def render(self):
        form = ServiceRowForm(self)
        return render_to_string(
            "services/item.html", {"object": self, "form": form}
        )

    @property
    def name(self):
        return self.item.name

    @property
    def code(self):
        return self.item.code

    @property
    def subtotal(self):
        return self.price * self.qty

    @property
    def api_url(self):
        return reverse_lazy("services:service", args=[self.pk])

    class Meta:
        db_table = "services_service"
        verbose_name = _("Service")
        verbose_name_plural = _("Service")



from django.contrib import admin

from services import models


@admin.register(models.Service)
class ServiceAdmin(admin.ModelAdmin):
    list_display = ["id", "created", "customer", "item", "price"]


@admin.register(models.ServiceItem)
class ServiceItemAdmin(admin.ModelAdmin):
    search_fields = ["name", "code"]
    list_display = ["id", "name", "code", "price"]


@admin.register(models.ServiceCategory)
class ServiceCategoryAdmin(admin.ModelAdmin):
    search_fields = ["name"]
    list_display = ["id", "name"]


@admin.register(models.ServiceWorker)
class ServiceWorkerAdmin(admin.ModelAdmin):
    search_fields = ["name"]
    list_display = ["id", "name"]



from django.db import models
from model_search import model_search
from tecdoc.utils import clean_code


class ServiceItemQuerySet(models.QuerySet):
    def search(self, code=None, category=None, query=None):
        queryset = self

        if category:
            queryset = queryset.filter(category=category)

        if code:
            queryset = queryset.filter(code__iexact=clean_code(code))

        if query:
            queryset = model_search(query, queryset, ["name"])

        return queryset



from datetime import datetime

from django import forms
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from djforms.fields import DatePickerField

from services.models import ServiceCategory, ServiceItem, ServiceWorker


class ReportForm(forms.Form):
    date_from = DatePickerField(label=_("Date from"))

    date_to = DatePickerField(label=_("Date to"))

    worker = forms.ModelChoiceField(
        label=_("worker"), required=False, queryset=ServiceWorker.objects.all()
    )

    def __init__(self, data):
        today = datetime.now().date().strftime(settings.DATE_INPUT_FORMATS[0])

        super().__init__(
            data={
                "date_from": data.get("date_from", today),
                "date_to": data.get("date_to", today),
                "worker": data.get("worker"),
            }
        )

        self.is_valid()


class SearchServiceForm(forms.Form):
    code = forms.CharField(required=False)

    query = forms.CharField(required=False)

    category = forms.ModelChoiceField(
        queryset=ServiceCategory.objects.all(),
        required=False,
        widget=forms.HiddenInput,
    )

    def clean(self):
        cleaned_data = {}

        for k, v in self.cleaned_data.items():
            if v:
                cleaned_data[k] = v

        return cleaned_data


class AddServiceItemForm(forms.ModelForm):
    class Meta:
        model = ServiceItem
        fields = ["category", "name", "code", "price"]


class ManageServiceForm(forms.Form):
    qty = forms.FloatField()

    price = forms.FloatField()

    worker = forms.ModelChoiceField(queryset=ServiceWorker.objects.all())



from django.utils.functional import SimpleLazyObject

from services.service import ServicesService


class ServicesMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        request.services = SimpleLazyObject(
            lambda: ServicesService(request.user)
        )
        return self.get_response(request)



# Generated by Django 3.0.13 on 2021-07-19 13:21

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("services", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="ServiceWorker",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        max_length=255, verbose_name="Worker name"
                    ),
                ),
            ],
            options={
                "verbose_name": "Service worker",
                "verbose_name_plural": "Service workers",
            },
        ),
        migrations.AlterModelTable(
            name="service",
            table="services_service",
        ),
        migrations.AddField(
            model_name="service",
            name="worker",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="services",
                to="services.ServiceWorker",
                verbose_name="Worker",
            ),
        ),
    ]



from django.urls import path

from services import views

app_name = "services"


urlpatterns = [
    path("report/", views.get_report, name="report"),
    path("items/", views.get_service_items, name="items"),
    path("add-to-sale/<int:sale_id>/", views.add_service, name="add"),
    path("<int:service_id>/", views.ServiceAPI.as_view(), name="service"),
    path(
        "print-services/<int:sale_id>/",
        views.print_services,
        name="print-services",
    ),
    path(
        "print-service-items/",
        views.print_service_items,
        name="print-service-items",
    ),
    path("add-service-item/", views.add_service_item, name="add-service-item"),
]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ServicesConfig(AppConfig):
    name = "services"
    verbose_name = _("Services")


default_app_config = "services.ServicesConfig"
